/** @file
    @brief  Definition of the internal implementation of the dt_node objects.
*/
#ifndef INCLUDED_DT_NODE_IMPL_H
#define INCLUDED_DT_NODE_IMPL_H

#include "dt_buf.h"
#include "dt_node.h"

/** The internal data structure used to represent each node type
*/
struct dt_node_ {
   dt_node_kind   kind;  /**< A value identifying the type of node */
   unsigned       refs;  /**< The use count (reference count) for this node. */

   /** The data contained by the node.
   Depending on the node kind, this data buffer will contain:
    - for leaf nodes,   the data associated with the node
    - for array nodes,  an array of node pointers ( an array of arr_item ).
    - for record nodes, an array of structures of type rec_item.
   */
   dt_buf  buf;
};
typedef struct dt_node_ dt_node;

/** The type of the entries stored in the buffer of an array node. */
typedef dt_node* arr_item;

/** The type of the entries stored in the buffer of a record node. */
typedef struct {
   dt_byte*   key;
   size_t     key_len;
   dt_node*   node;
 } rec_item;

/** An internal utility for checking the validity of a node */
dt_bool dt_node_is_valid( dt_node const* node );

#endif/*INCLUDED_DT_NODE_IMPL_H*/
