#ifndef INCLUDED_DT_STREAM_H
#define INCLUDED_DT_STREAM_H

#include "dt_type.h"

/** Identifier defining the contents of a dt_stream_step */
typedef enum {
   dt_leaf,    /**< Leaf data is provided in val and val_len. Key data is provided, if within a record. */
   dt_rec_in,  /**< Start contents of a record node. Only key data is provided, if within a record. */
   dt_rec_out, /**< End of a record block. No key or data information is provided. */
   dt_arr_in,  /**< Start contents of an array node. Only key data is provided, if within a record. */
   dt_arr_out, /**< End of a record block. No key or data information is provided. */
   dt_end      /**< Marks the end of the transmission of a data tree. No Key or data provided. */
} dt_step_kind;

/** Data providing all required data for a single step of the traversal of a data tree. */
typedef struct dt_stream_step_
{
   enum dt_step_kind kind;    /**< The type of this record step */
   dt_byte const*    key;     /**< the key data associated with this step. */
   size_t            key_len; /**< The length of the data eventually pointed to by key */
   dt_byte const*    val;     /**< the value data associated with a leaf step. */
   size_t            val_len; /**< The length of the data eventually pointed to by val */
} dt_stream_step;

typedef struct dt_reader_ dt_reader;
dt_bool dt_read ( dt_reader* obj, dt_stream_step* step);
void    dt_free_reader( dt_reader* obj );

typedef struct dt_writer_ dt_writer;
dt_bool dt_write(dt_writer* obj, dt_stream_step const* step);
void    dt_free_writer( dt_writer* obj );



#endif/*INCLUDED_DT_STREAM_H*/

