/** @file
    @brief  Utilities for converting an in-memory data tree to a stream.
*/
#ifndef INCLUDED_DT_STREAM_TREE_H
#define INCLUDED_DT_STREAM_TREE_H

#include "dt_stream.h"

/** Sends an existing in-memory data tree to a dt_writer object.
The tree @a src is traversed recursively, and information about its nodes are sent to @a dst.
An dt_end step is then sent to the writer.
The function exits immediately if the writer reports an error.
@return A non-zero value if the writer accepted the whole tree transmission without error.
*/
dt_bool dt_stream_tree( dt_node const* src, dt_writer* dst );


/** Function used internally by dt_stream_tree() to recursively send sub-nodes to the writer.
@param step Adress of an initialized dt_stream_step whose key fields have been pre-filled.
            The contents of this object may be overwritten upon return.
@param src  The node at the root of the sub-tree to be streamed.
@param dst  The stream writer to which the output will be sent.
*/
dt_bool dt_stream_subtree( dt_stream_step* step, dt_node const* src, dt_writer* dst );


/*struct dt_reader* dt_create_tree_reader( struct dt_node* tree ); */


/** Creates a writer object that will store the recived tree into *root.
@pre On entry, *root must be NULL (zero).
The writer will set it to a newly created node as soon as it starts receiving the tree.
@note  The user of the object has the responsibility of releasing the
       root node that will be stored in *root, at some point after the end of the transmission.
*/
dt_writer* dt_create_tree_writer( dt_node** root );


#endif/*INCLUDED_DT_STREAM_TREE_H*/

