/** @file
    @brief  A utility for verifying the validity of a data tree stream.
*/
#ifndef INCLUDED_DT_STREAM_VALIDATOR_H
#define INCLUDED_DT_STREAM_VALIDATOR_H

#include "dt_type.h"

/** Creates a writer that verifies the validity of the dt_step sequence it receives.
This writer can be used as a filter, which verifies that an input source
 (such as a file parser) provides a valid data structure.
@param delegate  Optional pointer to an existing dt_writer instance to which each dt_step
                 that has successfully been validated will be forwarded.
*/
dt_writer* dt_create_validator_writer( dt_writer* delegate );

/** Checks whether @a validator has received a dt_end step.
Because a datatree transmission may be truncated even though it does not contain an error,
this function can be called to verify that the transmission was complete and successful.
@return
   - A non-zero value if the transmission was complete and a dt_end step was received.
   - zero if the object has not received a complete transmission (or it is not a validator_writer).
*/
dt_bool dt_validator_writer_completed( dt_writer* validator );


#endif/*INCLUDED_DT_STREAM_VALIDATOR_H*/

