#include "dt_node.h"
#include "dt_stream.h"
#include "dt_stream_util.h"
#include "dt_value.h"
#include <stdio.h>
#include <stdlib.h>

int main(int argc, const char** argv)
{
   dt_node* tree = dt_read_tree_from_file( "test_src.txt" );
   if( ! tree )
      printf( "ERROR: The source file could not be read !\n" );
      
   {
      char* msg = 0;
      dt_read_str_dyn( dt_rec_sget( tree, "message" ), & msg );
      printf( "The arbitrarily long message is: %s\n", msg );
      free(msg);
   }
   {
      long i; unsigned long n; int b;
      char const* genderStrs[3] = { "male", "female", "<gender?>" };
      struct {
         char name[32];
         unsigned child_count;
         int balance;
         char married;
         char gender;  // 0=male,1=female,2=unknown
      } person;
      if( ! dt_read_strn( dt_rec_sget( tree, "name" ), person.name, sizeof(person.name) ) )
         sprintf( person.name, "<anonymous>" );
      if( dt_read_int( dt_rec_sget( tree, "balance" ), &i ) )
         person.balance = i;
      else // here we automatcially add a balance field
      {
         person.balance = 0;
         dt_rec_sput( tree, "balance", dt_make_int(person.balance) );
      }
      person.married = dt_read_bool( dt_rec_sget(tree, "married" ), &b ) ? b : 0;
      person.gender  = dt_read_bool( dt_rec_sget(tree, "female"  ), &b ) ? b : 2;
      person.child_count = dt_read_uint( dt_rec_sget(tree, "child_count" ), &n ) ? n : 0;
      printf( "'%s', %s, %s with %d children, has a balance of $ %d.\n"
            , person.name, genderStrs[person.gender]
            , person.married ? "married" : "single", person.child_count, person.balance );
   }
   
   {
      dt_bool writeOk = dt_write_tree_to_file( tree, "test_dst.txt" );
      printf( "File writing %s.\n", (writeOk?"was successful":"has FAILED") );
   }
   
   printf( "Now releasing the allocated tree structure.\n" );
   dt_node_release( tree );
   return 0;
}