/** @file
    @brief  Implementation of functions converting C types to/from datatree nodes (dt_node).
*/
#include "dt_value.h"

#include "dt_node.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>


dt_node* dt_make_bool( int i )
{
   return dt_make_str( i ? "YES" : "NO" );
}

dt_node* dt_make_int ( long i )
{
   char buf[32];
   sprintf(buf, "%li", i );
   return dt_make_str( buf );
}

dt_node* dt_make_uint( unsigned long u )
{
   char buf[32];
   sprintf(buf, "%lu", u );
   return dt_make_str( buf );
}

dt_node* dt_make_str ( const char* s )
{
   return dt_make_leaf( (dt_byte*)s, strlen(s) );
}

dt_node* dt_make_float( double f )
{
   char buf[64];
   sprintf(buf, "%g", f );
   return dt_make_str( buf );
}



dt_bool  dt_read_bool ( dt_node const* node, int*  b  )
{
   dt_byte* buf;
   size_t   len;
   if( dt_leaf_get_data(node,&buf,&len) )
   {
      if(   3 == len
         && tolower((int)buf[0])=='y'
         && tolower((int)buf[1])=='e'
         && tolower((int)buf[2])=='s'
         ) { *b = 1; return 1; }
      if(   2 == len
         && tolower((int)buf[0])=='n'
         && tolower((int)buf[1])=='o'
         ) { *b = 0; return 1; }
   }
   return 0;
}

dt_bool  dt_read_int  ( dt_node const* node, long* i )
{
   char buf[32];
   char* end;
   long v;
   if( ! dt_read_strn( node, buf, sizeof(buf) ) ) return 0;
   v = strtol( buf, &end, 10 );
   if( *end != '\0' ) return 0;
   *i = v;
   return 1;
}

dt_bool  dt_read_uint ( dt_node const* node, unsigned long* u )
{
   char buf[32];
   char* end;
   unsigned long v;
   if( ! dt_read_strn( node, buf, sizeof(buf) ) ) return 0;
   v = strtoul( buf, &end, 10 );
   if( *end != '\0' ) return 0;
   *u = v;
   return 1;
}

dt_bool  dt_read_float( dt_node const* node, double* f )
{
   char buf[64];
   char* end;
   double v;
   if( ! dt_read_strn( node, buf, sizeof(buf) ) ) return 0;
   v = strtod( buf, &end );
   if( *end != '\0' ) return 0;
   *f = v;
   return 1;
}


dt_bool  dt_read_strn   ( dt_node const* node, char* buf, size_t bufSize )
{
   char* strbuf;
   int i;
   dt_byte* binbuf;
   size_t   len;
   if(   ! dt_leaf_get_data(node,&binbuf,&len) 
      || len >= bufSize
      ) return 0;
   strbuf = (char*)binbuf;
   for( i = 0 ; i!=len ; ++i ) // reject if has null chars --> not a valid C string
      if(strbuf[i]=='\0') return 0;
   memcpy( buf, binbuf, len );
   buf[len] = '\0';
   return 1;
}

dt_bool  dt_read_str_dyn( dt_node const* node, char** s )
{
   char *strbuf, *dst;
   int i;
   dt_byte* binbuf;
   size_t   len;
   if( ! dt_leaf_get_data(node,&binbuf,&len) ) return 0;
   strbuf = (char*)binbuf;
   for( i = 0 ; i!=len ; ++i ) // reject if has null chars --> not a valid C string
      if(strbuf[i]=='\0') return 0;
   dst = (char*)realloc( *s, len+1 );
   if( dst == 0 ) return 0; // failed to allocate memory
   *s = dst;
   memcpy( dst, binbuf, len );
   dst[len] = '\0';
   return 1;
}




